/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@ricochet.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001-2004 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.ui.conf.jsp;

import java.io.IOException;
import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import org.apache.log4j.*;
import com.silverwrist.venice.ui.*;
import com.silverwrist.venice.ui.helpers.HTMLRendering;

public class PostBoxTag extends VeniceConfBodyTagSupport
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Logger logger = Logger.getLogger(PostBoxTag.class);

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private String m_action = null;
  private String m_type = "servlet";
  private int m_real_type = -1;
  private HashMap m_params = new HashMap();
  private String m_newtopic_name = null;
  private String m_newtopic_value = null;
  private String m_pseud_name = null;
  private String m_pseud_value = null;
  private String m_attach_name = null;
  private boolean m_attach_value = false;
  private String m_text_name = null;
  private String m_text_value = null;
  private ArrayList m_buttons = new ArrayList();

  /*--------------------------------------------------------------------------------
   * Overrides from class BodyTagSupport
   *--------------------------------------------------------------------------------
   */

  public int doStartTag() throws JspException
  {
    if (logger.isDebugEnabled())
      logger.debug("PostBoxTag.doStartTag(): entry");
    if (m_action==null)
      throw new JspTagException("<post:box/> action= attribute not specified!");
    HTMLRendering html = (HTMLRendering)(getRequestInput().queryService(HTMLRendering.class));
    m_real_type = html.convertLinkType(m_type);
    if (m_real_type<0)
      throw new JspTagException("<post:box/> type= attribute not a valid link type");
    return EVAL_BODY_BUFFERED;

  } // end doStartTag

  public int doAfterBody()
  {
    if (logger.isDebugEnabled())
      logger.debug("PostBoxTag.doAfterBody(): entry");
    return SKIP_BODY;

  } // end doAfterBody

  public int doEndTag() throws JspException
  {
    if (logger.isDebugEnabled())
      logger.debug("PostBoxTag.doEndTag(): entry");
    if (m_pseud_name==null)
      throw new JspTagException("<post:box/> tag has no <post:pseud/> tag inside it!");
    if (m_attach_name==null)
      throw new JspTagException("<post:box/> tag has no <post:attach/> tag inside it!");
    if (m_text_name==null)
      throw new JspTagException("<post:box/> tag has no <post:text/> tag inside it!");
    if (m_buttons.isEmpty())
      throw new JspTagException("<post:box/> tag has no <post:button/> tags inside it!");

    HTMLRendering html = (HTMLRendering)(getRequestOutput().queryService(HTMLRendering.class));

    try
    { // write out what we came here to accomplish
      JspWriter out = pageContext.getOut();
      out.write("<form method=\"POST\" action=\"" + html.formatURL(m_action,m_real_type)
		+ "\"><div class=\"content\">\n");
      Iterator it = m_params.entrySet().iterator();
      while (it.hasNext())
      { // write the hidden parameters
	Map.Entry ntry = (Map.Entry)(it.next());
	out.write("<input type=\"hidden\" name=\"" + ntry.getKey() + "\" value=\"" + ntry.getValue()
		  + "\" />\n");

      } // end while

      String my_font = html.getFontTag(html.CONTENT_FOREGROUND,"content");
      out.write("<table border=\"0\" cellpadding=\"0\">\n");
      if (m_newtopic_name!=null)
      { // write the "new topic name" field
	out.write("<tr><td align=\"left\" colspan=\"2\" class=\"content\">\n"
		  + my_font + "New topic name:</font><br />\n<span class=\"cinput\">"
		  + "<input type=\"text\" class=\"cinput\" name=\"" + m_newtopic_name
		  + "\" size=\"37\" maxlength=\"128\" VALUE=\"");
        if (m_newtopic_value!=null)
	  out.write(m_newtopic_value);
	out.write("\" /></span>\n</td></tr>\n");

      } // end if

      out.write("<tr><td align=\"left\" colspan=\"2\" class=\"content\">\n"
                + my_font + "Your name/header:</font><br/>\n<span class=\"cinput\">"
		+ "<input type=\"text\" class=\"cinput\" name=\"" + m_pseud_name
		+ "\" size=\"37\" maxlength=\"255\" value=\"");
      if (m_pseud_value!=null)
	out.write(m_pseud_value);
      out.write("\" /></span>\n" + my_font + "<input type=\"checkbox\" name=\"" + m_attach_name
		+ "\" value=\"Y\"");
      if (m_attach_value)
	out.write(" checked=\"checked\"");
      out.write(" /> Attach a file</font>\n</td></tr>\n<tr>\n<td align=\"left\" class=\"content\">\n"
		+ my_font + "Message:</font>\n</td>\n<td align=\"right\" class=\"content\">" + my_font
		+ "\n<a href=\"" + html.getExternalStaticPath("html-reference.html")
		+ "\" target=\"_blank\">HTML Guide</a>\n</font></td>\n</tr>\n<tr>"
		+ "<td align=\"left\" colspan=\"2\" class=\"cinput\">\n<textarea name=\"" + m_text_name
		+ "\" wrap=\"soft\" rows=\"7\" cols=\"51\">");
      if (m_text_value!=null)
	out.write(m_text_value);
      out.write("</textarea>\n</td></tr>\n<tr><td align=\"center\" colspan=\"2\" class=\"content\">\n");
      boolean first = true;
      it = m_buttons.iterator();
      while (it.hasNext())
      { // write out all the buttons
	String id = (String)(it.next());
	if (first)
	  first = false;
	else
	  out.write("&nbsp;");
	out.write(html.getButtonInput(id) + "\n");

      } // end while

      out.write("</td></tr>\n</table>\n</div></form>\n");

    } // end try
    catch (IOException e)
    { // convert the I/O error into something the servlet engine can deal with
      throw new JspTagException("error writing post box - " + e.getMessage());

    } // end catch

    // anything set by subtags must be cleared in doEndTag, not release
    m_params.clear();
    m_newtopic_name = null;
    m_newtopic_value = null;
    m_pseud_name = null;
    m_pseud_value = null;
    m_attach_name = null;
    m_attach_value = false;
    m_text_name = null;
    m_text_value = null;
    m_buttons.clear();
    return EVAL_PAGE;

  } // end doEndTag

  public void release()
  {
    if (logger.isDebugEnabled())
      logger.debug("PostBoxTag.release(): entry");
    super.release();
    m_action = null;
    m_type = "servlet";
    m_real_type = -1;

  } // end release

  /*--------------------------------------------------------------------------------
   * Attribute set functions
   *--------------------------------------------------------------------------------
   */

  public void setAction(String v)
  {
    m_action = v;

  } // end setAction

  public void setType(String v)
  {
    m_type = v;
    m_real_type = -1;

  } // end setType

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  final void addParam(String name, String value)
  {
    if (logger.isDebugEnabled())
      logger.debug("PostBoxTag.addParam(\"" + name + "\",\"" + value + "\") entry");
    m_params.put(name,value);

  } // end addParam

  final void setNewTopic(String param_name, String value) throws JspException
  {
    if (m_newtopic_name!=null)
      throw new JspTagException("<post:newtopic/> may only appear once inside a <post:box/>!");
    if (param_name==null)
      throw new JspTagException("<post:newtopic/> parameter name not specified!");
    m_newtopic_name = param_name;
    m_newtopic_value = value;

  } // end setNewTopic

  final void setPseud(String param_name, String value) throws JspException
  {
    if (logger.isDebugEnabled())
      logger.debug("PostBoxTag.setPseud(\"" + param_name + "\",\"" + value + "\") entry");
    if (m_pseud_name!=null)
      throw new JspTagException("<post:pseud/> may only appear once inside a <post:box/>!");
    if (param_name==null)
      throw new JspTagException("<post:pseud/> parameter name not specified!");
    m_pseud_name = param_name;
    m_pseud_value = value;

  } // end setPseud

  final void setAttach(String param_name, boolean value) throws JspException
  {
    if (m_attach_name!=null)
      throw new JspTagException("<post:attach/> may only appear once inside a <post:box/>!");
    if (param_name==null)
      throw new JspTagException("<post:attach/> parameter name not specified!");
    m_attach_name = param_name;
    m_attach_value = value;

  } // end setAttach

  final void setText(String param_name, String value) throws JspException
  {
    if (logger.isDebugEnabled())
      logger.debug("PostBoxTag.setText(\"" + param_name + "\",<value>) entry");
    if (m_text_name!=null)
      throw new JspTagException("<post:text/> may only appear once inside a <post:box/>!");
    if (param_name==null)
      throw new JspTagException("<post:text/> parameter name not specified!");
    m_text_name = param_name;
    m_text_value = value;

  } // end setText

  final void addButton(String id)
  {
    m_buttons.add(id);

  } // end addButton

} // end class PostBoxTag
